@@ -1,4 +1,5 @@
-# Copyright (C) 2015-2016 Jurriaan Bremer.
+# Copyright (C) 2015-2018 Jurriaan Bremer.
+# Copyright (C) 2018 Hatching B.V.
 # This file is part of SFlock - http://www.sflock.org/.
 # See the file 'docs/LICENSE.txt' for copying permission.
 
@@ -8,13 +9,14 @@ import tempfile
 
 from sflock.abstracts import Unpacker
 
+
 class RarFile(Unpacker):
     name = "rarfile"
     exe = "/usr/bin/rar"
-    exts = ".rar"
+    exts = b".rar"
     magic = "RAR archive"
 
-    def unpack(self, password=None, duplicates=None):
+    def unpack(self, password: str = None, duplicates=None):
         dirpath = tempfile.mkdtemp()
 
         if self.f.filepath:
@@ -24,15 +26,9 @@ class RarFile(Unpacker):
             filepath = self.f.temp_path()
             temporary = True
 
-        try:
-            subprocess.check_output([
-                self.zipjail, filepath, dirpath,
-                self.exe, "x", "-mt1", "-p%s" % (password or "-"),
-                filepath, dirpath,
-            ])
-        except subprocess.CalledProcessError as e:
-            self.f.mode = "failed"
-            self.f.error = e
+        ret = self.zipjail(filepath, dirpath, "x", "-mt1", "-p%s" % (password or "-"), filepath, dirpath)
+        if not ret:
+            return []
 
         if temporary:
             os.unlink(filepath)
